"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = Observable;

var _isObject = _interopRequireDefault(require("lodash/isObject"));

var _shallowequal = _interopRequireDefault(require("shallowequal"));

var _Watch = _interopRequireWildcard(require("./Watch"));

var _Notify = _interopRequireDefault(require("./Notify"));

var arrayMethods = ["push", "pop", "shift", "unshift", "splice", "sort", "reverse", "concat"];

function Observable(data, computed, isRoot) {
  if (computed === void 0) {
    computed = null;
  }

  if (isRoot === void 0) {
    isRoot = true;
  }

  var protoListener = {}; // 递归监控所有属性

  if (Array.isArray(data)) {
    data.forEach(function (value, i) {
      if ((0, _isObject.default)(value) || Array.isArray(value)) {
        data[i] = Observable(value, null, false);
      }
    });
  } else if ((0, _isObject.default)(data)) {
    Object.keys(data).forEach(function (key) {
      var value = data[key];

      if ((0, _isObject.default)(value) || Array.isArray(value)) {
        data[key] = Observable(value, null, false);
      }
    });
  }

  if (computed) {
    Object.keys(computed).forEach(function (key) {
      if (key in data) {
        console.warn("The computed property \"" + key + "\" is already defined in data.");
      }
    });
  }

  var handler = {
    get: function get(target, key, proxy) {
      if (key === "__ob__") return true;

      if (isRoot && key === "$watch") {
        return function (exp, watchFn) {
          var watcher = new _Watch.default(proxy, exp, watchFn);
          return function () {
            watcher.clearDeps();
          };
        };
      }

      var value = target[key];

      if (target.hasOwnProperty(key) && ((0, _isObject.default)(value) || Array.isArray(value)) && !value["__ob__"]) {
        value = target[key] = Observable(value, null, false);
      }

      var isComputed = computed && !(key in target) && (target.hasOwnProperty(key) || computed[key]) && key in computed && computed[key];

      if (_Watch.CurrentWatchDep.current && !isComputed) {
        var watcher = _Watch.CurrentWatchDep.current;
        protoListener[key] = protoListener[key] && protoListener[key] instanceof _Notify.default ? protoListener[key] : new _Notify.default();
        watcher.addDep(protoListener[key]);
      }

      if (Array.isArray(target) && arrayMethods.indexOf(key) !== -1) {
        return function () {
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          var ret = target[key].apply(target, args);
          protoListener[key] && protoListener[key].pub();
          return ret;
        };
      }

      return isComputed ? computed[key].call(proxy) : value;
    },
    set: function set(target, key, value) {
      var oldValue = target[key];
      target[key] = value;

      if (!(0, _shallowequal.default)(oldValue, value) && protoListener[key]) {
        protoListener[key].pub();
      }

      return true;
    }
  };
  return new Proxy(data, handler);
}