import isObject from "lodash/isObject";
import shallowequal from "shallowequal";
import Watch, { CurrentWatchDep } from "./Watch";
import Notify from "./Notify";
var arrayMethods = ["push", "pop", "shift", "unshift", "splice", "sort", "reverse", "concat"];
export default function Observable(data, computed, isRoot) {
  if (computed === void 0) {
    computed = null;
  }

  if (isRoot === void 0) {
    isRoot = true;
  }

  var protoListener = {}; // 递归监控所有属性

  if (Array.isArray(data)) {
    data.forEach(function (value, i) {
      if (isObject(value) || Array.isArray(value)) {
        data[i] = Observable(value, null, false);
      }
    });
  } else if (isObject(data)) {
    Object.keys(data).forEach(function (key) {
      var value = data[key];

      if (isObject(value) || Array.isArray(value)) {
        data[key] = Observable(value, null, false);
      }
    });
  }

  if (computed) {
    Object.keys(computed).forEach(function (key) {
      if (key in data) {
        console.warn("The computed property \"" + key + "\" is already defined in data.");
      }
    });
  }

  var handler = {
    get: function get(target, key, proxy) {
      if (key === "__ob__") return true;

      if (isRoot && key === "$watch") {
        return function (exp, watchFn) {
          var watcher = new Watch(proxy, exp, watchFn);
          return function () {
            watcher.clearDeps();
          };
        };
      }

      var value = target[key];

      if (target.hasOwnProperty(key) && (isObject(value) || Array.isArray(value)) && !value["__ob__"]) {
        value = target[key] = Observable(value, null, false);
      }

      var isComputed = computed && !(key in target) && (target.hasOwnProperty(key) || computed[key]) && key in computed && computed[key];

      if (CurrentWatchDep.current && !isComputed) {
        var watcher = CurrentWatchDep.current;
        protoListener[key] = protoListener[key] && protoListener[key] instanceof Notify ? protoListener[key] : new Notify();
        watcher.addDep(protoListener[key]);
      }

      if (Array.isArray(target) && arrayMethods.indexOf(key) !== -1) {
        return function () {
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          var ret = target[key].apply(target, args);
          protoListener[key] && protoListener[key].pub();
          return ret;
        };
      }

      return isComputed ? computed[key].call(proxy) : value;
    },
    set: function set(target, key, value) {
      var oldValue = target[key];
      target[key] = value;

      if (!shallowequal(oldValue, value) && protoListener[key]) {
        protoListener[key].pub();
      }

      return true;
    }
  };
  return new Proxy(data, handler);
}